###########################################################
# AUTHOR  : Carsten Bttemeier (@spmoshpit), 
#		    based on a script by @hicannl 
# DATE    : 02-04-2014 
# COMMENT : This script creates new Active Directory users
#           including different kind of properties based
#           on an input_create_ad_users.csv.
###########################################################
Import-Module ActiveDirectory
# Get current directory and set import file in variable
$path     = Split-Path -parent $MyInvocation.MyCommand.Definition
$newpath  = $path + "\create_ad_users_spmoshpit.csv"
$pathUtf8 = $path + "\import_create_ad_users_spmoshpit_utf8.csv"
# Define variables
$log      = $path + "\create_ad_users.log"
$date     = Get-Date
$i        = 0
#Set Password for user accounts
$password = "123456"
# Change this to the location you want the users to be created in your AD
$ouParent = "DC=spmoshpit,DC=dev"
$location = "OU=spmoshpit,DC=spmoshpit,DC=dev"
$locationService = "OU=Serviceaccounts,DC=spmoshpit,DC=dev"

# FUNCTIONS
Function createOUs
{
	[string] $Path = $location
	try {
		if (!([adsi]::Exists("LDAP://$Path"))) {
			New-ADOrganizationalUnit -Name spmoshpit -Path $ouParent 
		} 
	} catch {
		# If invalid format, error is thrown.
		Throw("Supplied Path is invalid.`n$_")
	}

	[string] $Path = $locationService
	try {
		if (!([adsi]::Exists("LDAP://$Path"))) {
			New-ADOrganizationalUnit -Name Serviceaccounts -Path $ouParent 
		} 
	} catch {
		# If invalid format, error is thrown.
		Throw("Supplied Path is invalid.`n$_")
	}
}

Function createUsers
{
  "Created following users (on " + $date + "): " | Out-File $log -append
  "--------------------------------------------" | Out-File $log -append
  # parse the input file to UCS2-Litte endian
  Get-Content $newpath | Out-File $pathUtf8 -Encoding Unicode
  # import utf8 csv
  Import-CSV $pathUtf8 | ForEach-Object { 
    # A check for the country, because those were full names and need 
    # to be landcodes in order for AD to accept them. 
    If($_.CO -eq "Germany")
    {
      $_.CO = "DE"
    }
    # Replace dots / points (.) in names, because AD will error when a 
    # name ends with a dot (and it looks cleaner as well)
    $replace = $_.CN.Replace(".","")
    If($replace.length -lt 4)
    {
      $lastname = $replace
    }
    Else
    {
      $lastname = $replace.substring(0,4)
    }
    # Create sAMAccountName according to this 'naming convention':
    $sam = $_.CN
    $dn = "CN=" + $sam + ",CN=Users," + $ouParent

    Try   { $exists = Get-ADUser -LDAPFilter "(sAMAccountName=$sam)" } 
    Catch { } 
    If(!$exists) 
    { 
      $i++ 
      # Set all variables according to the table names in the Excel  
      # sheet / import CSV. The names can differ in every project, but  
      # if the names change, make sure to change it below as well. 
      $setpass = ConvertTo-SecureString -AsPlainText -force $password
      New-ADUser $sam -GivenName $_.GivenName -Initials $_.Initials -Surname $_.SN -DisplayName $_.DisplayName -Office $_.physicalDeliveryOfficeName -Description $_.Description -EmailAddress $_.Mail -StreetAddress $_.StreetAddress -City $_.L -PostalCode $_.PostalCode -Country $_.CO -UserPrincipalName $_.UPN -Company $_.Company -Department $_.Department -EmployeeID $_.ID -Title $_.Title -OfficePhone $_.OfficePhone -HomePhone $_.HomePhone -MobilePhone $_.MobilePhone -Fax $_.Fax -State $_.State -POBox $_.POBox -AccountPassword $setpass Enabled $true -PasswordNeverExpires $true
      
      #set manager
      If (-not [string]::isnullorempty($_.Manager))
      {
	write-host $_.Manager
        Get-ADUser $sam | Set-ADUser -Manager $_.Manager
      }
  
      # Move the user to the OU you set above. If you don't want to 
      # move the user(s) and just create them in the global Users 
      # OU, comment the string below 
	  If ($_.IsService)
	  {
        Move-ADObject -Identity $dn -TargetPath $locationService 
	  }
	  else
	  {
        Move-ADObject -Identity $dn -TargetPath $location
	  }
  
      # Rename the object to a good looking name (otherwise you see 
      # the 'ugly' shortened sAMAccountNames as a name in AD. This  
      # can't be set right away (as sAMAccountName) due to the 20 
      # character restriction 
      $newdn = (Get-ADUser $sam).DistinguishedName 
      Rename-ADObject -Identity $newdn -NewName $_.CN 
  
      $output  = $i.ToString() + ") Name: " + $_.CN + "  sAMAccountName: "  
      $output += $sam + "  Pass: " + $_.Password 
      $output | Out-File $log -append 
    } 
    Else 
    { 
      "SKIPPED - ALREADY EXISTS OR ERROR: " + $_.CN | Out-File $log -append 
    } 
  } 
  "----------------------------------------" + "`n" | Out-File $log -append 
} 
# RUN SCRIPT 
createOUs
createUsers 
#Finished